#========================================================================================
# package      DE_EPAGES::Ipayment::t::CGI_Billsafe_Negative_si
# state        public
#----------------------------------------------------------------------------------------
# description  System Integration Test for Billsafe Transactions
#========================================================================================
package DE_EPAGES::Ipayment::t::CGI_Billsafe_Negative_si;
use base DE_EPAGES::Shop::API::TestImportXML;

use strict;

use Readonly;
use URI;
use URI::QueryParam;

use DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth;
use DE_EPAGES::Ipayment::API::Order::Payment;
use DE_EPAGES::Ipayment::API::Session;
use DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath );

use Test::More;
use Test::MockObject::Extends;


#----------------------------------------------------------------------------------------
# Wrap for DE_EPAGES::Shop::API::TestImportXML
# You need to override this methods in order to use
#----------------------------------------------------------------------------------------
sub shopAlias {
    return 'CgiBillsafeShop';
}
sub importXMLPath {
    return 'ShopXMLs/ShopWithIpaymentBillsafe.xml';
}

#----------------------------------------------------------------------------------------

sub test {
    my $self = shift;

    my $Order;
    Readonly my $ORDERPATH => $self->shopPath().'/Customers/TestCustomer/Orders/1234-5678';

    # Transaction With Invalid Ipayment Credentials
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _testTransactionForInvalidCredentials($Order);

    # Transaction With Invalid Billsafe specific parameters (Birthdate & Gender)
    $Order = LoadObjectByPath($ORDERPATH);
    _createSession($Order);
    _testTransactionForInvalidBillsafeParameters($Order);

    done_testing();
}
#----------------------------------------------------------------------------------------

sub _createSession() {
    my $SubTestName = (caller(0))[3];
    my ($Order) = @_;

    #prepare
    my $Session = DE_EPAGES::Ipayment::API::Session->new();
    $Session = Test::MockObject::Extends->new( $Session );
    DisableDoubleTransactionCheck($Session);

    #request
    my $Status = $Session->createSessionAtIpaymentAndStoreIdInDatabase($Order);

SKIP: {
    skip( 'Temporarily disabled b/c of https://epages.atlassian.net/browse/AD-1119.');

    ok($Status, "$SubTestName - tests if the session has be created");
    my $SessionID = $Order->container()->get('Payment')->get('SessionID');
    ok(isValid($SessionID), "$SubTestName - tests if the session is stored in the database");
}
}



sub DisableDoubleTransactionCheck {
    my ($Session ) = @_;

    $Session->mock('_createHashbuilder', sub {
        my $self = shift;
        my ($DatabaseBasketOrOrder) = @_;
        my $HashBuilder = DE_EPAGES::Ipayment::API::SOAP::Request::HashBuilder->new($DatabaseBasketOrOrder);
        my $MockedHashBuilder = Test::MockObject::Extends->new( $HashBuilder );

        $MockedHashBuilder->mock( '_assembleOptions', sub {
            my $self = shift;
            return {
                    'fromIp' => '',
                    'advancedStrictIdCheck' => '0',
                    'checkFraudattack' => '',
                    'checkDoubleTrx' => '0',
                    'errorLang' => '',
                    'trxLongsafe' => '',
                    'otherOptions' => {
                        'option' => {
                            'key' => '',
                            'value' => ''
                        }
                    },
                    'browserData' => {
                        'browserUserAgent' => '',
                        'browserAcceptHeaders' => '',
                    },
                    'clientData' => {
                        'clientName' => 'ePages',
                        'clientVersion' => '',
                    }
                };
        } );
        return $MockedHashBuilder;
    });
    return undef;
}

sub _testTransactionForInvalidCredentials() {
    my ($Order) = @_;

    my $SubTestName = (caller(0))[3];

    my $hHackedRequest = {
        'account_id'  => '',
        'trxuser_id'  => '',
        'trxpassword' => '',
    };

    my $ResponseParameters = _transactionWithOptionalHackedRequestData($Order, $hHackedRequest);

    is($ResponseParameters->{'ret_status'}, 'ERROR">Silent-Redirect</a><br', "$SubTestName - test return status (Error)");

}


sub _testTransactionForInvalidBillsafeParameters() {
    my ($Order) = @_;

    my $SubTestName = (caller(0))[3];

    my $hHackedRequest = {
        'addr_birthday'  => '',
        'addr_gender'  => '',
    };

    my $ResponseParameters = _transactionWithOptionalHackedRequestData($Order, $hHackedRequest);

    is($ResponseParameters->{'ret_status'}, 'ERROR">Silent-Redirect</a><br', "$SubTestName - test return status (Error)");
}

sub _transactionWithOptionalHackedRequestData() {
    my ($Order, $hHackedRequest) = @_;

    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    my $ReturnParams = $AuthOrPreauth->getRequestParameters($Order);
    my $hRequestParams = $ReturnParams->{'HiddenInputs'};
    my $PaymentUrl = $ReturnParams->{'PaymentUrl'};

    foreach my $key (keys %{$hHackedRequest}) {
        $hRequestParams->{$key} = $hHackedRequest->{$key};
    }

    my $UserAgent = LWP::UserAgent->new();
    my $Response = $UserAgent->post( $PaymentUrl, $hRequestParams );

    my $ReturnUrl = $Response->{'_content'};
    my $RedirectURI = URI->new($ReturnUrl);
    my $ResponseParameters = $RedirectURI->query_form_hash();

    return $ResponseParameters;
}

__PACKAGE__->runScript();

1;
